@@ -421,7 +421,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
             sgensx1 = [_af_new(_) for _ in sgensx]
             deltab = _orbit(size, sgensx1, b)
         else:
-            deltab = set([b])
+            deltab = {b}
         # p1 = min(IMAGES) = min(Union D_p*h*deltab for h in TAB)
         if all_metrics_with_sym:
             md = _min_dummies(dumx, sym, indices)
@@ -970,7 +970,7 @@ def get_transversals(base, gens):
         return []
     stabs = _distribute_gens_by_base(base, gens)
     orbits, transversals = _orbits_transversals_from_bsgs(base, stabs)
-    transversals = [dict((x, h._array_form) for x, h in y.items()) for y in
+    transversals = [{x: h._array_form for x, h in y.items()} for y in
                     transversals]
     return transversals
 
