@@ -1,18 +1,19 @@
 from __future__ import print_function, division
 
-from sympy.core import S, sympify, Dummy
+from sympy.core import S, sympify, Dummy, Mod
 from sympy.core.function import Function, ArgumentIndexError
 from sympy.core.logic import fuzzy_and
-from sympy.core.numbers import Integer
+from sympy.core.numbers import Integer, pi
+from sympy.core.relational import Eq
+
 from sympy.ntheory import sieve
+
 from math import sqrt as _sqrt
 
-from sympy.core.compatibility import reduce, range
+from sympy.core.compatibility import reduce, range, HAS_GMPY
 from sympy.core.cache import cacheit
 
-from sympy.polys.polytools import poly_from_expr
-from sympy.polys.polyerrors import PolificationFailed
-
+from sympy.polys.polytools import Poly
 
 class CombinatorialFunction(Function):
     """Base class for combinatorial functions. """
@@ -43,7 +44,7 @@ class factorial(CombinatorialFunction):
        kind is very useful in case of combinatorial simplification.
 
        Computation of the factorial is done using two algorithms. For
-       small arguments naive product is evaluated. However for bigger
+       small arguments a precomputed look up table is used. However for bigger
        input algorithm Prime-Swing is used. It is the fastest algorithm
        known and computes n! via prime factorization of special class
        of numbers, called here the 'Swing Numbers'.
@@ -91,6 +92,8 @@ class factorial(CombinatorialFunction):
         35102025, 5014575, 145422675, 9694845, 300540195, 300540195
     ]
 
+    _small_factorials = []
+
     @classmethod
     def _swing(cls, n):
         if n < 33:
@@ -147,20 +150,23 @@ class factorial(CombinatorialFunction):
                 if n.is_negative:
                     return S.ComplexInfinity
                 else:
-                    n, result = n.p, 1
+                    n = n.p
 
                     if n < 20:
-                        for i in range(2, n + 1):
-                            result *= i
+                        if not cls._small_factorials:
+                            result = 1
+                            for i in range(1, 20):
+                                result *= i
+                                cls._small_factorials.append(result)
+                        result = cls._small_factorials[n-1]
+
+                    # GMPY factorial is faster, use it when available
+                    elif HAS_GMPY:
+                        from sympy.core.compatibility import gmpy
+                        result = gmpy.fac(n)
+
                     else:
-                        N, bits = n, 0
-
-                        while N != 0:
-                            if N & 1 == 1:
-                                bits += 1
-
-                            N = N >> 1
-
+                        bits = bin(n).count('1')
                         result = cls._recursive(n)*2**(n - bits)
 
                     return Integer(result)
@@ -325,9 +331,10 @@ class factorial2(CombinatorialFunction):
     @classmethod
     def eval(cls, arg):
         # TODO: extend this to complex numbers?
+
         if arg.is_Number:
-            if arg.is_infinite:
-                return
+            if not arg.is_Integer:
+                raise ValueError("argument must be nonnegative integer or negative odd integer")
 
             # This implementation is faster than the recursive one
             # It also avoids "maximum recursion depth exceeded" runtime error
@@ -335,13 +342,13 @@ class factorial2(CombinatorialFunction):
                 if arg.is_even:
                     k = arg / 2
                     return 2 ** k * factorial(k)
-
                 return factorial(arg) / factorial2(arg - 1)
 
-            if arg.is_even:
-                raise ValueError("argument must be nonnegative or odd")
 
-            return arg * (S.NegativeOne) ** ((1 - arg) / 2) / factorial2(-arg)
+            if arg.is_odd:
+                return arg * (S.NegativeOne) ** ((1 - arg) / 2) / factorial2(-arg)
+            raise ValueError("argument must be nonnegative integer or negative odd integer")
+
 
     def _eval_is_even(self):
         # Double factorial is even for every positive even input
@@ -388,6 +395,10 @@ class factorial2(CombinatorialFunction):
             if n.is_odd:
                 return ((n + 1) / 2).is_even
 
+    def _eval_rewrite_as_gamma(self, n):
+        from sympy import gamma, Piecewise, sqrt
+        return 2**(n/2)*gamma(n/2 + 1) * Piecewise((1, Eq(Mod(n, 2), 0)), (sqrt(2/pi), Eq(Mod(n, 2), 1)))
+
 
 ###############################################################################
 ######################## RISING and FALLING FACTORIALS ########################
@@ -395,7 +406,8 @@ class factorial2(CombinatorialFunction):
 
 
 class RisingFactorial(CombinatorialFunction):
-    """Rising factorial (also called Pochhammer symbol) is a double valued
+    """
+    Rising factorial (also called Pochhammer symbol) is a double valued
     function arising in concrete mathematics, hypergeometric functions
     and series expansions. It is defined by:
 
@@ -405,15 +417,16 @@ class RisingFactorial(CombinatorialFunction):
     more information check "Concrete mathematics" by Graham, pp. 66
     or visit http://mathworld.wolfram.com/RisingFactorial.html page.
 
-    When x is a polynomial f of a single variable y of order >= 1,
-    rf(x,k) = f(y) * f(y+1) * ... * f(x+k-1) as described in
-    Peter Paule, "Greatest Factorial Factorization and Symbolic Summation",
-    Journal of Symbolic Computation, vol. 20, pp. 235-268, 1995.
+    When x is a Poly instance of degree >= 1 with a single variable,
+    rf(x,k) = x(y) * x(y+1) * ... * x(y+k-1), where y is the variable of x.
+    This is as described in Peter Paule, "Greatest Factorial Factorization and
+    Symbolic Summation", Journal of Symbolic Computation, vol. 20, pp.
+    235-268, 1995.
 
     Examples
     ========
 
-    >>> from sympy import rf, symbols, factorial, ff, binomial
+    >>> from sympy import rf, symbols, factorial, ff, binomial, Poly
     >>> from sympy.abc import x
     >>> n, k = symbols('n k', integer=True)
     >>> rf(x, 0)
@@ -422,7 +435,7 @@ class RisingFactorial(CombinatorialFunction):
     120
     >>> rf(x, 5) == x*(1 + x)*(2 + x)*(3 + x)*(4 + x)
     True
-    >>> rf(x**3, 2)
+    >>> rf(Poly(x**3, x), 2)
     Poly(x**6 + 3*x**5 + 3*x**4 + x**3, x, domain='ZZ')
 
     Rewrite
@@ -443,6 +456,7 @@ class RisingFactorial(CombinatorialFunction):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Pochhammer_symbol
+
     """
 
     @classmethod
@@ -467,37 +481,34 @@ class RisingFactorial(CombinatorialFunction):
                         else:
                             return S.Infinity
                     else:
-                        try:
-                            F, opt = poly_from_expr(x)
-                        except PolificationFailed:
-                            return reduce(lambda r, i: r*(x + i), range(0, int(k)), 1)
-                        if len(opt.gens) > 1 or F.degree() <= 1:
-                            return reduce(lambda r, i: r*(x + i), range(0, int(k)), 1)
+                        if isinstance(x, Poly):
+                            gens = x.gens
+                            if len(gens)!= 1:
+                                raise ValueError("rf only defined for polynomials on one generator")
+                            else:
+                                return reduce(lambda r, i:
+                                              r*(x.shift(i).expand()),
+                                              range(0, int(k)), 1)
                         else:
-                            v = opt.gens[0]
-                            return reduce(lambda r, i:
-                                          r*(F.subs(v, v + i).expand()),
-                                          range(0, int(k)), 1)
+                            return reduce(lambda r, i: r*(x + i), range(0, int(k)), 1)
+
                 else:
                     if x is S.Infinity:
                         return S.Infinity
                     elif x is S.NegativeInfinity:
                         return S.Infinity
                     else:
-                        try:
-                            F, opt = poly_from_expr(x)
-                        except PolificationFailed:
-                            return 1/reduce(lambda r, i:
-                                            r*(x - i),
-                                            range(1, abs(int(k)) + 1), 1)
-                        if len(opt.gens) > 1 or F.degree() <= 1:
-                            return 1/reduce(lambda r, i:
-                                            r*(x - i),
-                                            range(1, abs(int(k)) + 1), 1)
+                        if isinstance(x, Poly):
+                            gens = x.gens
+                            if len(gens)!= 1:
+                                raise ValueError("rf only defined for polynomials on one generator")
+                            else:
+                                return 1/reduce(lambda r, i:
+                                                r*(x.shift(-i).expand()),
+                                                range(1, abs(int(k)) + 1), 1)
                         else:
-                            v = opt.gens[0]
                             return 1/reduce(lambda r, i:
-                                            r*(F.subs(v, v - i).expand()),
+                                            r*(x - i),
                                             range(1, abs(int(k)) + 1), 1)
 
     def _eval_rewrite_as_gamma(self, x, k):
@@ -525,7 +536,8 @@ class RisingFactorial(CombinatorialFunction):
 
 
 class FallingFactorial(CombinatorialFunction):
-    """Falling factorial (related to rising factorial) is a double valued
+    """
+    Falling factorial (related to rising factorial) is a double valued
     function arising in concrete mathematics, hypergeometric functions
     and series expansions. It is defined by
 
@@ -535,12 +547,13 @@ class FallingFactorial(CombinatorialFunction):
     more information check "Concrete mathematics" by Graham, pp. 66
     or visit http://mathworld.wolfram.com/FallingFactorial.html page.
 
-    When x is a polynomial f of a single variable y of order >= 1,
-    ff(x,k) = f(y) * f(y-1) * ... * f(x-k+1) as described in
-    Peter Paule, "Greatest Factorial Factorization and Symbolic Summation",
-    Journal of Symbolic Computation, vol. 20, pp. 235-268, 1995.
+    When x is a Poly instance of degree >= 1 with single variable,
+    ff(x,k) = x(y) * x(y-1) * ... * x(y-k+1), where y is the variable of x.
+    This is as described in Peter Paule, "Greatest Factorial Factorization and
+    Symbolic Summation", Journal of Symbolic Computation, vol. 20, pp.
+    235-268, 1995.
 
-    >>> from sympy import ff, factorial, rf, gamma, polygamma, binomial, symbols
+    >>> from sympy import ff, factorial, rf, gamma, polygamma, binomial, symbols, Poly
     >>> from sympy.abc import x, k
     >>> n, m = symbols('n m', integer=True)
     >>> ff(x, 0)
@@ -549,7 +562,7 @@ class FallingFactorial(CombinatorialFunction):
     120
     >>> ff(x, 5) == x*(x-1)*(x-2)*(x-3)*(x-4)
     True
-    >>> ff(x**2, 2)
+    >>> ff(Poly(x**2, x), 2)
     Poly(x**4 - 2*x**3 + x**2, x, domain='ZZ')
     >>> ff(n, n)
     factorial(n)
@@ -574,6 +587,7 @@ class FallingFactorial(CombinatorialFunction):
     ==========
 
     .. [1] http://mathworld.wolfram.com/FallingFactorial.html
+
     """
 
     @classmethod
@@ -598,18 +612,16 @@ class FallingFactorial(CombinatorialFunction):
                         else:
                             return S.Infinity
                     else:
-                        try:
-                            F, opt = poly_from_expr(x)
-                        except PolificationFailed:
-                            return reduce(lambda r, i: r*(x - i),
-                                          range(0, int(k)), 1)
-                        if len(opt.gens) > 1 or F.degree() <= 1:
-                            return reduce(lambda r, i: r*(x - i),
-                                          range(0, int(k)), 1)
+                        if isinstance(x, Poly):
+                            gens = x.gens
+                            if len(gens)!= 1:
+                                raise ValueError("ff only defined for polynomials on one generator")
+                            else:
+                                return reduce(lambda r, i:
+                                              r*(x.shift(-i).expand()),
+                                              range(0, int(k)), 1)
                         else:
-                            v = opt.gens[0]
-                            return reduce(lambda r, i:
-                                          r*(F.subs(v, v - i).expand()),
+                            return reduce(lambda r, i: r*(x - i),
                                           range(0, int(k)), 1)
                 else:
                     if x is S.Infinity:
@@ -617,18 +629,16 @@ class FallingFactorial(CombinatorialFunction):
                     elif x is S.NegativeInfinity:
                         return S.Infinity
                     else:
-                        try:
-                            F, opt = poly_from_expr(x)
-                        except PolificationFailed:
-                            return 1/reduce(lambda r, i: r*(x + i),
-                                            range(1, abs(int(k)) + 1), 1)
-                        if len(opt.gens) > 1 or F.degree() <= 1:
-                            return 1/reduce(lambda r, i: r*(x + i),
-                                            range(1, abs(int(k)) + 1), 1)
+                        if isinstance(x, Poly):
+                            gens = x.gens
+                            if len(gens)!= 1:
+                                raise ValueError("rf only defined for polynomials on one generator")
+                            else:
+                                return 1/reduce(lambda r, i:
+                                                r*(x.shift(i).expand()),
+                                                range(1, abs(int(k)) + 1), 1)
                         else:
-                            v = opt.gens[0]
-                            return 1/reduce(lambda r, i:
-                                            r*(F.subs(v, v + i).expand()),
+                            return 1/reduce(lambda r, i: r*(x + i),
                                             range(1, abs(int(k)) + 1), 1)
 
     def _eval_rewrite_as_gamma(self, x, k):
@@ -845,6 +855,9 @@ class binomial(CombinatorialFunction):
         from sympy import gamma
         return gamma(n + 1)/(gamma(k + 1)*gamma(n - k + 1))
 
+    def _eval_rewrite_as_tractable(self, n, k):
+        return self._eval_rewrite_as_gamma(n, k).rewrite('tractable')
+
     def _eval_rewrite_as_FallingFactorial(self, n, k):
         if k.is_integer:
             return ff(n, k) / factorial(k)
