@@ -11,6 +11,8 @@ from sympy.functions import (
 from sympy.core.compatibility import range
 from sympy.utilities.pytest import XFAIL, raises
 
+from sympy.core.numbers import GoldenRatio
+
 x = Symbol('x')
 
 
@@ -68,6 +70,8 @@ def test_fibonacci():
     assert fibonacci(n).rewrite(sqrt) == \
         2**(-n)*sqrt(5)*((1 + sqrt(5))**n - (-sqrt(5) + 1)**n) / 5
     assert fibonacci(n).rewrite(sqrt).subs(n, 10).expand() == fibonacci(10)
+    assert fibonacci(n).rewrite(GoldenRatio).subs(n,10).evalf() == \
+        fibonacci(10)
     assert lucas(n).rewrite(sqrt) == \
         (fibonacci(n-1).rewrite(sqrt) + fibonacci(n+1).rewrite(sqrt)).simplify()
     assert lucas(n).rewrite(sqrt).subs(n, 10).expand() == lucas(10)
