@@ -2,7 +2,8 @@ from __future__ import print_function, division
 
 from .matexpr import MatrixExpr
 from sympy import Basic, sympify
-
+from sympy.matrices import Matrix
+from sympy.functions.elementary.complexes import re, im
 
 class FunctionMatrix(MatrixExpr):
     """
@@ -45,3 +46,6 @@ class FunctionMatrix(MatrixExpr):
     def _eval_trace(self):
         from sympy.matrices.expressions.trace import Trace
         return Trace._eval_rewrite_as_Sum(Trace(self)).doit()
+
+    def as_real_imag(self):
+        return (re(Matrix(self)), im(Matrix(self)))
