@@ -1,14 +1,26 @@
-from sympy import Mul, S, Pow, Symbol, summation, factorial as fac
+from sympy import (Sieve, binomial_coefficients, binomial_coefficients_list,
+    Mul, S, Pow, sieve, Symbol, summation, Dummy,
+    factorial as fac)
 from sympy.core.numbers import Integer, Rational
 from sympy.core.compatibility import long, range
 
-from sympy.ntheory import totient, factorint, primefactors, divisors, nextprime, pollard_rho, \
-    perfect_power, multiplicity, trailing, divisor_count, primorial, pollard_pm1, divisor_sigma
+from sympy.ntheory import (isprime, n_order, is_primitive_root,
+    is_quad_residue, legendre_symbol, jacobi_symbol, npartitions, totient,
+    factorint, primefactors, divisors, randprime, nextprime, prevprime,
+    primerange, primepi, prime, pollard_rho, perfect_power, multiplicity,
+    trailing, divisor_count, primorial, pollard_pm1, divisor_sigma,
+    factorrat, reduced_totient)
+from sympy.ntheory.factor_ import (smoothness, smoothness_p,
+    antidivisors, antidivisor_count, core, digits, udivisors, udivisor_sigma,
+    udivisor_count, primenu, primeomega)
+from sympy.ntheory.generate import cycle_length
+from sympy.ntheory.multinomial import (
+    multinomial_coefficients, multinomial_coefficients_iterator)
+from sympy.ntheory.bbp_pi import pi_hex_digits
+from sympy.ntheory.modular import crt, crt1, crt2, solve_congruence
+
+from sympy.utilities.pytest import raises, slow
 
-from sympy.ntheory.factor_ import smoothness, smoothness_p, \
-    antidivisors, antidivisor_count, core, digits, udivisors, udivisor_sigma, \
-    udivisor_count
-from sympy.utilities.pytest import raises
 from sympy.utilities.iterables import capture
 
 
@@ -142,6 +154,20 @@ def test_factorint():
     assert factorint(5951757) == {3: 1, 7: 1, 29: 2, 337: 1}
     assert factorint(64015937) == {7993: 1, 8009: 1}
     assert factorint(2**(2**6) + 1) == {274177: 1, 67280421310721: 1}
+
+    assert factorint(0, multiple=True) == [0]
+    assert factorint(1, multiple=True) == []
+    assert factorint(-1, multiple=True) == [-1]
+    assert factorint(-2, multiple=True) == [-1, 2]
+    assert factorint(-16, multiple=True) == [-1, 2, 2, 2, 2]
+    assert factorint(2, multiple=True) == [2]
+    assert factorint(24, multiple=True) == [2, 2, 2, 3]
+    assert factorint(126, multiple=True) == [2, 3, 3, 7]
+    assert factorint(123456, multiple=True) == [2, 2, 2, 2, 2, 2, 3, 643]
+    assert factorint(5951757, multiple=True) == [3, 7, 29, 29, 337]
+    assert factorint(64015937, multiple=True) == [7993, 8009]
+    assert factorint(2**(2**6) + 1, multiple=True) == [274177, 67280421310721]
+
     assert multiproduct(factorint(fac(200))) == fac(200)
     for b, e in factorint(fac(150)).items():
         assert e == fac_multiplicity(150, b)
@@ -264,7 +290,7 @@ def test_udivisors_and_udivisor_count():
 
 def test_issue_6981():
     S = set(divisors(4)).union(set(divisors(Integer(2))))
-    assert S == set([1,2,4])
+    assert S == {1,2,4}
 
 
 def test_totient():
@@ -283,6 +309,22 @@ def test_totient():
     n = Symbol("n", integer=True, positive=True)
     assert totient(n).is_integer
 
+def test_reduced_totient():
+    assert [reduced_totient(k) for k in range(1, 16)] == \
+        [1, 1, 2, 2, 4, 2, 6, 2, 6, 4, 10, 2, 12, 6, 4]
+    assert reduced_totient(5005) == 60
+    assert reduced_totient(5006) == 2502
+    assert reduced_totient(5009) == 5008
+    assert reduced_totient(2**100) == 2**98
+
+    m = Symbol("m", integer=True)
+    assert reduced_totient(m)
+    assert reduced_totient(m).subs(m, 2**3*3**10) == 3**10 - 3**9
+    assert summation(reduced_totient(m), (m, 1, 16)) == 68
+
+    n = Symbol("n", integer=True, positive=True)
+    assert reduced_totient(n).is_integer
+
 
 def test_divisor_sigma():
     assert [divisor_sigma(k) for k in range(1, 12)] == \
@@ -390,6 +432,20 @@ def test_visual_factorint():
     assert -1 in factorint(-42, visual=True).args
 
 
+def test_factorrat():
+    assert str(factorrat(S(12)/1, visual=True)) == '2**2*3**1'
+    assert str(factorrat(S(1)/1, visual=True)) == '1'
+    assert str(factorrat(S(25)/14, visual=True)) == '5**2/(2*7)'
+    assert str(factorrat(S(-25)/14/9, visual=True)) == '-5**2/(2*3**2*7)'
+
+    assert factorrat(S(12)/1, multiple=True) == [2, 2, 3]
+    assert factorrat(S(1)/1, multiple=True) == []
+    assert factorrat(S(25)/14, multiple=True) == [1/7, 1/2, 5, 5]
+    assert factorrat(S(12)/1, multiple=True) == [2, 2, 3]
+    assert factorrat(S(-25)/14/9, multiple=True) == \
+        [-1, 1/7, 1/3, 1/3, 1/2, 5, 5]
+
+
 def test_visual_io():
     sm = smoothness_p
     fi = factorint
@@ -447,3 +503,31 @@ def test_digits():
     assert digits(384753, 71) == [71, 1, 5, 23, 4]
     assert digits(93409) == [10, 9, 3, 4, 0, 9]
     assert digits(-92838, 11) == [-11, 6, 3, 8, 2, 9]
+
+
+def test_primenu():
+    assert primenu(2) == 1
+    assert primenu(2 * 3) == 2
+    assert primenu(2 * 3 * 5) == 3
+    assert primenu(3 * 25) == primenu(3) + primenu(25)
+    assert [primenu(p) for p in primerange(1, 10)] == [1, 1, 1, 1]
+    assert primenu(fac(50)) == 15
+    assert primenu(2 ** 9941 - 1) == 1
+    n = Symbol('n', integer=True)
+    assert primenu(n)
+    assert primenu(n).subs(n, 2 ** 31 - 1) == 1
+    assert summation(primenu(n), (n, 2, 30)) == 43
+
+
+def test_primeomega():
+    assert primeomega(2) == 1
+    assert primeomega(2 * 2) == 2
+    assert primeomega(2 * 2 * 3) == 3
+    assert primeomega(3 * 25) == primeomega(3) + primeomega(25)
+    assert [primeomega(p) for p in primerange(1, 10)] == [1, 1, 1, 1]
+    assert primeomega(fac(50)) == 108
+    assert primeomega(2 ** 9941 - 1) == 1
+    n = Symbol('n', integer=True)
+    assert primeomega(n)
+    assert primeomega(n).subs(n, 2 ** 31 - 1) == 1
+    assert summation(primeomega(n), (n, 2, 30)) == 59
