@@ -5,9 +5,9 @@ from collections import defaultdict
 from sympy import SYMPY_DEBUG
 
 from sympy.core.evaluate import global_evaluate
-from sympy.core.compatibility import iterable, ordered, as_int, default_sort_key
+from sympy.core.compatibility import iterable, ordered, default_sort_key
 from sympy.core import expand_power_base, sympify, Add, S, Mul, Derivative, Pow, symbols, expand_mul
-from sympy.core.numbers import Rational, Float
+from sympy.core.numbers import Rational
 from sympy.core.exprtools import Factors, gcd_terms
 from sympy.core.mul import _keep_coeff, _unevaluated_Mul
 from sympy.core.function import _mexpand
@@ -16,7 +16,6 @@ from sympy.functions import exp, sqrt, log
 from sympy.polys import gcd
 from sympy.simplify.sqrtdenest import sqrtdenest
 
-import mpmath
 
 
 
@@ -388,7 +387,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
             # none of the patterns matched
             disliked += product
     # add terms now for each key
-    collected = dict([(k, Add(*v)) for k, v in collected.items()])
+    collected = {k: Add(*v) for k, v in collected.items()}
 
     if disliked is not S.Zero:
         collected[S.One] = disliked
@@ -629,7 +628,7 @@ def collect_const(expr, *vars, **kwargs):
 
 
 def radsimp(expr, symbolic=True, max_terms=4):
-    """
+    r"""
     Rationalize the denominator by removing square roots.
 
     Note: the expression returned from radsimp must be used with caution
@@ -652,8 +651,6 @@ def radsimp(expr, symbolic=True, max_terms=4):
     >>> from sympy.simplify.radsimp import collect_sqrt
     >>> from sympy.abc import a, b, c
 
-    >>> radsimp(1/(I + 1))
-    (1 - I)/2
     >>> radsimp(1/(2 + sqrt(2)))
     (-sqrt(2) + 2)/2
     >>> x,y = map(Symbol, 'xy')
@@ -736,14 +733,14 @@ def radsimp(expr, symbolic=True, max_terms=4):
         if not d.is_Pow:
             return False
         e = d.exp
-        if e.is_Rational and e.q == 2 or symbolic and fraction(e)[1] == 2:
+        if e.is_Rational and e.q == 2 or symbolic and denom(e) == 2:
             return True
         if log2:
             q = 1
             if e.is_Rational:
                 q = e.q
             elif symbolic:
-                d = fraction(e)[1]
+                d = denom(e)
                 if d.is_Integer:
                     q = d
             if q != 1 and log(q, 2).is_Integer:
@@ -773,7 +770,7 @@ def radsimp(expr, symbolic=True, max_terms=4):
             return expr
 
         if ispow2(d):
-            d2 = sqrtdenest(sqrt(d.base))**fraction(d.exp)[0]
+            d2 = sqrtdenest(sqrt(d.base))**numer(d.exp)
             if d2 != d:
                 return handle(1/d2)
         elif d.is_Pow and (d.exp.is_integer or d.base.is_positive):
@@ -937,7 +934,7 @@ def fraction(expr, exact=False):
        flag is unset, then structure this exponent's structure will
        be analyzed and pretty fraction will be returned:
 
-       >>> from sympy import exp
+       >>> from sympy import exp, Mul
        >>> fraction(2*x**(-y))
        (2, x**y)
 
@@ -947,6 +944,14 @@ def fraction(expr, exact=False):
        >>> fraction(exp(-x), exact=True)
        (exp(-x), 1)
 
+       The `exact` flag will also keep any unevaluated Muls from
+       being evaluated:
+
+       >>> u = Mul(2, x + 1, evaluate=False)
+       >>> fraction(u)
+       (2*x + 2, 1)
+       >>> fraction(u, exact=True)
+       (2*(x  + 1), 1)
     """
     expr = sympify(expr)
 
@@ -958,6 +963,11 @@ def fraction(expr, exact=False):
             if ex.is_negative:
                 if ex is S.NegativeOne:
                     denom.append(b)
+                elif exact:
+                    if ex.is_constant():
+                        denom.append(Pow(b, -ex))
+                    else:
+                        numer.append(term)
                 else:
                     denom.append(Pow(b, -ex))
             elif ex.is_positive:
@@ -974,8 +984,10 @@ def fraction(expr, exact=False):
             denom.append(d)
         else:
             numer.append(term)
-
-    return Mul(*numer), Mul(*denom)
+    if exact:
+        return Mul(*numer, evaluate=False), Mul(*denom, evaluate=False)
+    else:
+        return Mul(*numer), Mul(*denom)
 
 
 def numer(expr):
