@@ -1,4 +1,4 @@
-"""
+r"""
 This module is intended for solving recurrences or, in other words,
 difference equations. Currently supported are linear, inhomogeneous
 equations with polynomial or rational coefficients.
@@ -21,7 +21,7 @@ where `k > 0` and `a_{i}(n)` are polynomials in `n`. To use
 
     ``L = [ a_{0}(n), ..., a_{k-1}(n), a_{k}(n) ]``
 
-where ``L[i]``, for `i=0, \dots, k`, maps to
+where ``L[i]``, for `i=0, \ldots, k`, maps to
 `a_{i}(n) y(n+i)` (`y(n+i)` is implicit).
 
 For example if we would like to compute `m`-th Bernoulli polynomial
@@ -69,7 +69,7 @@ from sympy.utilities.iterables import numbered_symbols
 
 
 def rsolve_poly(coeffs, f, n, **hints):
-    """
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order
     `k` with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f`, where `f` is a polynomial, we seek for
@@ -347,7 +347,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
 
 def rsolve_ratio(coeffs, f, n, **hints):
-    """
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order `k`
     with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f`, where `f` is a polynomial, we seek
@@ -458,7 +458,7 @@ def rsolve_ratio(coeffs, f, n, **hints):
 
 
 def rsolve_hyper(coeffs, f, n, **hints):
-    """
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order `k`
     with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f` we seek for all hypergeometric solutions
@@ -653,23 +653,23 @@ def rsolve_hyper(coeffs, f, n, **hints):
         return None
 
     if hints.get('symbols', False):
-        symbols |= set([s for s, k in sk])
+        symbols |= {s for s, k in sk}
         return (result, list(symbols))
     else:
         return result
 
 
 def rsolve(f, y, init=None):
-    """
+    r"""
     Solve univariate recurrence with rational coefficients.
 
     Given `k`-th order linear recurrence `\operatorname{L} y = f`,
     or equivalently:
 
     .. math:: a_{k}(n) y(n+k) + a_{k-1}(n) y(n+k-1) +
-              \dots + a_{0}(n) y(n) = f(n)
+              \cdots + a_{0}(n) y(n) = f(n)
 
-    where `a_{i}(n)`, for `i=0, \dots, k`, are polynomials or rational
+    where `a_{i}(n)`, for `i=0, \ldots, k`, are polynomials or rational
     functions in `n`, and `f` is a hypergeometric function or a sum
     of a fixed number of pairwise dissimilar hypergeometric terms in
     `n`, finds all solutions or returns ``None``, if none were found.
@@ -683,7 +683,7 @@ def rsolve(f, y, init=None):
 
         ``L = [ v_0, v_1, ..., v_m ]``
 
-    where ``L[i] = v_i``, for `i=0, \dots, m`, maps to `y(n_i)`.
+    where ``L[i] = v_i``, for `i=0, \ldots, m`, maps to `y(n_i)`.
 
     Examples
     ========
@@ -801,7 +801,7 @@ def rsolve(f, y, init=None):
 
     if symbols and init is not None:
         if type(init) is list:
-            init = dict([(i, init[i]) for i in range(len(init))])
+            init = {i: init[i] for i in range(len(init))}
 
         equations = []
 
